/*
 * This is the header for function prototypes,
 * constants, and included headers, that are
 * commonly used by saltine cracker.
 *
 * Copyright (C) 1999 aempirei
 * Produced by The Ambient Empire
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <time.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <netdb.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/uio.h>
#include <unistd.h>
#include <sys/wait.h>
#include <sys/un.h>

#ifdef HAVE_SYS_SELECT_H
#include <sys/select.h>
#endif

#ifdef HAVE_POLL_H
#include <poll.h>
#endif

#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif

#define TRUE       1
#define FALSE      0
#define KEDS_PORT  1984
#define KEYSPACES  46656L
#define NTHASHLEN  32
#define C3HASHLEN  13
#define NTHASHBLEN 16
#define C3HASHBLEN 8

#define SA struct sockaddr
#define IPV4_ADDRLEN 16

extern long *crypt3(char *, char *);
extern long *nthash(char *);
extern long *c3atob(char *);
extern long *ntatob(char *);

long flen(FILE *);
int test_integrity();

const char *ipv4_ntop(int, const void *, char *, size_t);
const char *ipv4_showname(int , const void *, char *, size_t);

long flen(FILE *fs) {
   long i, j;
   j = ftell(fs); fseek(fs, 0L, SEEK_END);
   i = ftell(fs); fseek(fs,  j, SEEK_SET);
   return i;
}

int test_integrity() {
   printf("- LibDES Crypt(3) integrity: ");
   if(memcmp(crypt3("art", "aa"), c3atob("CrNbjlN7/RI"), C3HASHBLEN)) {
      fprintf(stderr, "Checksum FAILED\n");
      return(-1);
   } else printf("Checksum PASSED\n");

   printf("- MD4 NT Hash integrity: ");
   if(memcmp(nthash("aaaa"), ntatob("FA5664875FFADF0AF61ABF9B097FA46F"), NTHASHBLEN)) {
      fprintf(stderr, "Checksum FAILED\n");
      return(-1);
   } else printf("Checksum PASSED\n");
   
   return(0);
}

const char *
ipv4_ntop(int family, const void *addrptr, char *strptr, size_t len) {
   const u_char *p = (const u_char *)addrptr;

   if(family == AF_INET) {
      char temp[IPV4_ADDRLEN];

      snprintf(temp, sizeof(temp), "%d.%d.%d.%d", p[0], p[1], p[2], p[3]);
      if(strlen(temp) >= len) {
         errno = ENOSPC;
         return(NULL);
      }
      strcpy(strptr, temp);
      return(strptr);
   }
   errno = EAFNOSUPPORT;
   return(NULL);
}

const char *
ipv4_showname(int family, const void *addrptr, char *strptr, size_t len) {
   struct hostent *hentry;
   
   size_t aflen;

   if(family == AF_INET) aflen = 4;
   else {
      errno = EAFNOSUPPORT;
      return(NULL);
   }
   
   if((hentry = gethostbyaddr(addrptr, aflen, family)) != NULL) {
      if(strlen(hentry->h_name) < len) {
         strcpy(strptr, hentry->h_name);
         return(strptr);
      }
   }

   return(ipv4_ntop(family, addrptr, strptr, len));
}

int ipv4_rslvhost(char *hostname, struct sockaddr_in *hostinfo) {
   struct hostent *hentry;
   
   if(!inet_aton(hostname, &hostinfo->sin_addr)) {
      if((hentry = gethostbyname(hostname)) != NULL) {
         memcpy(&hostinfo->sin_addr, hentry->h_addr, hentry->h_length);
      } else return(0);
   }

   return(1);
}

